@@ -1,27 +1,30 @@
-use lazy_static::lazy_static;
 use num::ToPrimitive;
 use num_rational::Ratio;
 use paste::paste;
 
+use std::borrow::Cow;
 use std::fmt;
 
-use crate::parse::token::Field as TokField;
-use crate::parse::token::Value as TokValue;
+use crate::parse::token::Field as ParsedField;
+use crate::parse::token::Value as ParsedValue;
 
 #[derive(Clone, PartialEq, Debug)]
-pub enum Token {
-    Field(Field),
-    Comment { is_inline: bool, inner: String },
+pub enum Token<'a> {
+    Field(Field<'a>),
+    Comment {
+        is_inline: bool,
+        inner: Cow<'a, str>,
+    },
     Checksum(u8),
 }
 
-impl<'a, 'input: 'a> From<&'a TokField<'input>> for Token {
-    fn from(field: &'a TokField<'input>) -> Self {
+impl<'input> From<&ParsedField<'input>> for Token<'input> {
+    fn from(field: &ParsedField<'input>) -> Self {
         Self::Field(field.into())
     }
 }
 
-impl fmt::Display for Token {
+impl fmt::Display for Token<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         use Token::*;
         match self {
@@ -37,28 +40,28 @@ impl fmt::Display for Token {
 
 /// Fundamental unit of GCode: a value preceded by a descriptive letter.
 #[derive(Clone, PartialEq, Debug)]
-pub struct Field {
-    pub letters: String,
-    pub value: Value,
+pub struct Field<'a> {
+    pub letters: Cow<'a, str>,
+    pub value: Value<'a>,
 }
 
-impl fmt::Display for Field {
+impl<'a> fmt::Display for Field<'a> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{}{}", self.letters, self.value)
     }
 }
 
-impl<'a, 'input: 'a> From<&'a TokField<'input>> for Field {
-    fn from(field: &'a TokField<'input>) -> Self {
+impl<'input> From<&ParsedField<'input>> for Field<'input> {
+    fn from(field: &ParsedField<'input>) -> Self {
         Self {
-            letters: field.letters.to_string(),
+            letters: field.letters.into(),
             value: Value::from(&field.value),
         }
     }
 }
 
-impl From<Field> for Token {
-    fn from(field: Field) -> Token {
+impl<'a> From<Field<'a>> for Token<'a> {
+    fn from(field: Field<'a>) -> Token<'a> {
         Token::Field(field)
     }
 }
@@ -66,14 +69,14 @@ impl From<Field> for Token {
 /// All the possible variations of a field's value.
 /// Some flavors of GCode also allow for strings.
 #[derive(Clone, PartialEq, Debug)]
-pub enum Value {
+pub enum Value<'a> {
     Rational(Ratio<i64>),
     Float(f64),
     Integer(usize),
-    String(String),
+    String(Cow<'a, str>),
 }
 
-impl Value {
+impl Value<'_> {
     pub fn as_f64(&self) -> Option<f64> {
         match self {
             Self::Rational(r) => r.to_f64(),
@@ -84,18 +87,18 @@ impl Value {
     }
 }
 
-impl<'a, 'input: 'a> From<&'a TokValue<'input>> for Value {
-    fn from(val: &'a TokValue<'input>) -> Self {
-        use TokValue::*;
+impl<'input> From<&ParsedValue<'input>> for Value<'input> {
+    fn from(val: &ParsedValue<'input>) -> Self {
+        use ParsedValue::*;
         match val {
             Rational(r) => Self::Rational(*r),
             Integer(i) => Self::Integer(*i),
-            String(s) => Self::String(s.to_string()),
+            String(s) => Self::String(Cow::Borrowed(s)),
         }
     }
 }
 
-impl fmt::Display for Value {
+impl fmt::Display for Value<'_> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::Rational(r) => write!(f, "{}", r.to_f64().ok_or(fmt::Error)?),
@@ -119,7 +122,7 @@ macro_rules! command {
                 [<$commandName:snake:lower>](
                     vec![$(
                         Field {
-                            letters: stringify!([<$arg:upper>]).to_string(),
+                            letters: Cow::Borrowed(stringify!([<$arg:upper>])),
                             value: Value::Float($value),
                         }
                     ,)*].drain(..)
@@ -135,7 +138,7 @@ macro_rules! impl_commands {
         paste! {
             $(
                 $(#[$outer])*
-                pub fn [<$commandName:snake:lower>]<I: Iterator<Item = Field>>(args: I) -> Command {
+                pub fn [<$commandName:snake:lower>]<'a, I: Iterator<Item = Field<'a>>>(args: I) -> Command<'a> {
                     Command {
                         name: [<$commandName:snake:upper _FIELD>].clone(),
                         args: args.filter(|arg| {
@@ -146,26 +149,23 @@ macro_rules! impl_commands {
                         }).collect(),
                     }
                 }
-
-                lazy_static! {
-                    pub static ref [<$commandName:snake:upper _FIELD>]: Field = Field {
-                        letters: $letters.to_string(),
-                        value:Value::Integer($value),
-                    };
-                }
+                pub const [<$commandName:snake:upper _FIELD>]: Field<'static> = Field {
+                    letters: Cow::Borrowed($letters),
+                    value: Value::Integer($value),
+                };
             )*
         }
 
         /// Commands are the operational unit of GCode
         /// They consist of a G, M, or other top-level field followed by field arguments
         #[derive(Clone, PartialEq, Debug)]
-        pub struct Command {
-            name: Field,
-            args: Vec<Field>,
+        pub struct Command<'a> {
+            name: Field<'a>,
+            args: Vec<Field<'a>>,
         }
 
-        impl Command {
-            pub fn push(&mut self, arg: Field) {
+        impl<'a> Command<'a> {
+            pub fn push(&mut self, arg: Field<'a>) {
                 match &self.name {
                     $(x if *x == paste!{[<$commandName:snake:upper _FIELD>]}.clone() => {
                         if match arg.letters.to_ascii_uppercase().as_str() {
@@ -187,7 +187,7 @@ macro_rules! impl_commands {
                 std::iter::once(&self.name).chain(self.args.iter())
             }
 
-            pub fn into_token_vec(mut self) -> Vec<Token> {
+            pub fn into_token_vec(mut self) -> Vec<Token<'a>> {
                 std::iter::once(self.name).chain(self.args.drain(..)).map(|f| f.into()).collect()
             }
 
@@ -195,7 +195,7 @@ macro_rules! impl_commands {
                 self.iter().skip(1)
             }
 
-            pub fn iter_mut_args(&mut self) -> impl Iterator<Item = &mut Field> {
+            pub fn iter_mut_args(&mut self) -> impl Iterator<Item = &mut Field<'a>> {
                 self.args.iter_mut()
             }
 
@@ -204,7 +204,7 @@ macro_rules! impl_commands {
                 self.iter_args().find(|arg| arg.letters == letters)
             }
 
-            pub fn set(&mut self, letters: &str, value: Value) {
+            pub fn set(&mut self, letters: &str, value: Value<'a>) {
                 let letters = letters.to_ascii_uppercase();
                 for i in 0..self.args.len() {
                     if self.args[i].letters == letters {
